/* bsp14.q: mehrspurige Aufnahme+Wiedergabe */

import midi, mididev;

def (_,IN,_) = MIDIDEV!0, (_,OUT,PORT) = MIDIDEV!0,
  PLAY = midi_open "bsp14 - play", REC = midi_open "bsp14 - rec",
  _ = midi_connect IN REC || midi_connect REC PLAY || midi_connect PLAY OUT;

def _ = midi_accept_type REC active_sense false ||
  midi_accept_type REC clock false;

/* Eingabeschleife */

recloop K S T L (_,_,_,stop)
		= midi_send REC PORT stop || reverse L;
recloop K S T L (_,_,T1,MSG)
		= midi_send PLAY PORT MSG ||
		  recloop K S1 T1 [(K,S1,MSG)|L] (midi_get REC)
		    where D = time_diff T T1, S1 = S+D;

time_diff T1 T2	= ifelse (D>=0) D (D+0x100000000)
		    where D = T2 mod 0x100000000 - T1 mod 0x100000000;

/* Wiedergabeschleife */

playloop _ _ SEQ
		= SEQ if midi_avail PLAY and then (midi_get PLAY!3=stop);
playloop S T []	= [];
playloop S T [(K,S,MSG)|SEQ]
		= midi_send PLAY PORT MSG || print (K,T,MSG) ||
		  playloop S T SEQ;
playloop S T SEQ
		= playloop S1 T1 SEQ
		    where [(_,S1,_)|_] = SEQ, D = time_diff S S1,
		      T1 = midi_wait PLAY (T+D);

print (K,T,MSG)	= printf "%10d: %3d:  %s\n" (T,K,str MSG);

/* Aufnahme */

public record SEQ;

record K []	= midi_flush REC || recloop K S T [] (midi_get REC)
  where S = 0, T = midi_time;

record K SEQ	= midi_flush PLAY || playloop S T SEQ || result TH
  where [(_,S,_)|_] = SEQ, T = midi_time,
    TH = midi_flush REC || thread (recloop K S T [] (midi_get REC));

/* Wiedergabe */

public play SEQ;

play []		= [];
play SEQ	= midi_flush PLAY || playloop S T SEQ
		    where [(_,S,_)|_] = SEQ, T = midi_time;

/* Mischen zweier MIDI-Sequenzen */

public mix SEQ1 SEQ2;

mix SEQ1 SEQ2	= SEQ1 if null SEQ2;
		= SEQ2 if null SEQ1;
		= [hd SEQ1|mix (tl SEQ1) SEQ2] if T1 <= T2
		    where (_,T1,_) = hd SEQ1, (_,T2,_) = hd SEQ2;
		= [hd SEQ2|mix SEQ1 (tl SEQ2)] otherwise;

/* Extraktion einer Spur */

public track K;

track K SEQ	= filter (trackeq K) SEQ;

trackeq K (K,_,_)
		= true;
trackeq _ _	= false otherwise;
