/* mididev.q: portable MIDI-Gertedefinitionen */

import midi;

/* **************************************************************************
   BEGINN der Konfigurationsinformationen. Die folgenden Definitionen mssen
   ggf. an die lokalen Systemeinstellungen angepasst werden.
   ************************************************************************* */

/* 1. Allgemeine Konfigurations-Informationen -- NUR FR LINUX-SYSTEME. */

/* Standard-Midi-Gert zur Verwendung mit msRawMidi. */

def DEV_MIDI	= "/dev/midi";

/* Sample-Rate und SoundFont-Datei fr iiwusynth. */

def SAMPLE_RATE = 22050, SOUND_FONT = "/usr/share/sfbank/creative/default.sf2";

/* Host-Tabelle. Geben Sie hier alle Hostnamen des Netzwerks an, zu denen eine
   Verbindung hergestellt werden soll. */

def HOSTS	= ["localhost"];

/* Gruppen- und Latenzzeit (in ms) fr den msInetDriver. Die Default-Werte
   sind normalerweise fr ein lokales Netz ausreichend. Falls es Probleme mit
   der Synchronisierung im Netzwerk gibt, mssen diese Werte ggf. erhht
   werden. Fr genauere Erluterungen siehe `man msInetDriver'. */

def GROUP_TIME = 10, LATENCY = 10;

/* 2. Autostart der Gertetreiber unter Linux. Dies ist nur erforderlich,
   falls die Gertetreiber nicht anderweitig gestartet werden. */

def STARTUP	= startup;

startup		= midi_raw_driver || midi_synth_driver ||
		  midi_inet_driver HOSTS if pos "linux" sysinfo >= 0;
		= () otherwise;

/* 3. MIDI-Gerte-Tabelle. Jeder Eintrag gibt den Namen eines Gerts an
   zusammen mit der Referenznummer des MidiShare-Clients und der Portnummer,
   ber die das Gert angesprochen werden kann. Standardmig werden drei
   Gerte definiert: "midi" (externe MIDI-Schnittstelle), "synth" (interner
   Synthesizer) und "inet" (Netzwerk). Bei Bedarf fgen Sie weitere Gerte-
   Beschreibungen am Ende der Tabelle hinzu.

   WINDOWS: Stimmen Sie die Port-Nummern mit den durch das msDrivers-Programm
   vorgenommenen Einstellungen ab.

   LINUX: Fr den Netzwerk-Client wird eine symbolische Konstante angegeben,
   die von midi_connect und midi_disconnect automatisch in die aktuelle Liste
   der Host-Clients aufgelst wird (s.u.). */

public var MIDIDEV;
public const midi_inet_ref;

def MIDIDEV	= mididev;

mididev

= /* LINUX */

[("midi",midi_client_ref "/dev/midi",0),
 ("synth",midi_client_ref "iiwusynth",0),
 ("inet",midi_inet_ref,0)]

if pos "linux" sysinfo >= 0;

= /* WINDOWS */

[("midi",0,0),
 ("synth",0,1),
 ("inet",0,2)]

otherwise;

/* **************************************************************************
   ENDE der Konfigurationsinformationen. Editieren der folgenden Definitionen
   ist normalerweise nicht erforderlich.
   ************************************************************************* */

/* Erweiterung der midi_connect und midi_disconnect Funktionen, um dynamisch
   Netzwerk-Verbindungen unter Linux einzurichten. */

midi_connect REF midi_inet_ref
		= do (midi_connect REF) (map midi_inet_client HOSTS);
midi_connect midi_inet_ref REF
		= do (flip midi_connect REF) (map midi_inet_client HOSTS);

midi_disconnect REF midi_inet_ref
		= do (midi_disconnect REF) (map midi_inet_client HOSTS);
midi_disconnect midi_inet_ref REF
		= do (flip midi_disconnect REF) (map midi_inet_client HOSTS);

/* MidiShare-Treiberprogramme fr Linux */

/* Die Ein- und Ausgabe ber die externe MIDI-Schnittstelle erledigt das
   msRawMidi-Programm. */

midi_raw_driver	= midi_client DEV_MIDI
		  (sprintf
		   "xterm -iconic -e msRawMidi %s &"
		   DEV_MIDI);

/* Zur Ausgabe ber die Soundkarte wird iiwusynth verwendet (erhltlich unter
   http://www.iiwu.org/iiwusynth). Auerdem bentigen Sie eine SoundFont-
   Datei, deren Namen in der Variablen SOUND_FONT anzugeben ist. */

midi_synth_driver
		= midi_client "iiwusynth"
		  (sprintf
		   "xterm -iconic -e iiwusynth -r %d -m midishare %s &"
		   (SAMPLE_RATE,SOUND_FONT));

/* Ein- und Ausgabe ber das Netzwerk erfolgt ber msInetDriver. Fr jeden der
   angegebenen Hostnamen richtet der Treiber einen entsprechenden MidiShare-
   Client ein. */

midi_inet_driver HOSTS
		= midi_client "msWANDriver"
		  (sprintf
		   "xterm -iconic -e msInetDriver -lat %d -g %d -s %s &"
		   (LATENCY,GROUP_TIME,join " " HOSTS));

/* Zur Kommunikation mit einem Host muss mit dem entsprechenden Client eine
   Verbindung hergestellt werden. Der Name des Client entspricht dem
   Hostnamen, mglicherweise ergnzt um die Domain-Suffix (host.domain). Die
   Referenznummer des Client zu einem gegebenen Host liefert die folgende
   Funktion. */

midi_inet_client NAME
		= REF where REF:Int = midi_client_ref NAME;
		= REF where PREF = NAME++".",
			[REF:Int|_] =
			filter (lambda N (pos PREF (midi_client_name N)=0))
			midi_clients;
